/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/********************************************************************
* Author: Virtualik <virty1k@mail.ru>
* WWW   : http://virtualsoft.narod.ru
* Copyright (c) 2003,2006 Virtualik  All rights reserved.
********************************************************************/

#include "config.h"
#include <stdio.h>
#include <time.h>

#ifdef _Windows
#include <mmsystem.h>
#else
#include "sndplay.h"
char *devicename;
int device;
#endif

char *soundpath;
char f_accurate = 1;
char f_quiet,
  f_cut12,
  fasttime = 0;

#define passes(l,h) (curtime->tm_min <= h && curtime->tm_min >= l)

void usage ()
{
#ifdef EXTPLAYER
 printf ("Compiled to use with <%s>\n", EXTPLAYER);
#endif
 puts ("Usage: talktime [options]");
 puts ("Available options:");
#ifndef EXTPLAYER
 puts ("\t-d (device) = set DSP device (default /dev/dsp)");
#endif
 printf ("\t-f = Time without words of hours and minutes\n");
 printf ("\t-s (path) = set path with sound (default %s)\n", DEFSOUNDPATH);
 puts ("\t-i = Talk inaccurate time (with stripping to 10 minutes accuracy)");
 puts ("\t-c = Use 12-hours mode");
 puts ("\t-q = Be quiet. Don't prints copyrights message");
 puts ("\t-h = Show this help");
}

int main (int argc, char *argv[])
{
 int ch;
 time_t lt = time (NULL);
 struct tm *curtime;
 char szFilename[128] = { 0 };

 soundpath = (char *) malloc (PATH_MAX);
 memset (soundpath, 0, PATH_MAX);
 //Parsing command line parameters
 while ((ch = getopt (argc, argv, "cdihfqs:")) != -1)
  switch (ch)
    {
#ifndef EXTPLAYER
    case 'd':
     devicename = optarg;
     break;
#endif
    case 'c':
     f_cut12 = 1;
     break;
    case 'i':
     f_accurate = 0;
     break;
    case 's':
     strcpy (soundpath, optarg);
     break;
    case 'f':
     fasttime = 1;
     break;
    case 'q':
     f_quiet = 1;
     break;
    case 'h':
    default:
     usage ();
     return 0;
    }
 argc -= optind;
 argv += optind;

 // Print program info
 if (!f_quiet)
   {
    printf ("Time Talker for %s version %s\n",
#ifdef _Windows
	    "MS Windows"
#else
	    "Unix/Linux"
#endif
	    , VERSION);
    puts ("Written and copyrigts by Vitaly Checkryzhev <virty1k@mail.ru>");
    puts
     ("THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY! USE AT YOUR OWN RISK!");
    puts ("Get more help by calling talktime with -h flag");
   }

#ifndef EXTPLAYER
 // Default dsp device
 if (devicename == NULL)
  devicename = DEV_DSP;

 // Try to open device
 if (PrepareDSP (devicename) != 0)
   {
    printf ("Error opening DSP [%s]\n", devicename);
    goto alldone;
   }
#endif

 // Default sound path
 if (strlen (soundpath) == 0)
  strcat (soundpath, DEFSOUNDPATH);

 // Add slash to the end
 if (soundpath[strlen (soundpath)] != '/')
  strncat (soundpath, "/", 1);

 // Get current local time
 curtime = (struct tm *) localtime (&lt);

 if (f_cut12 == 1)
   {
    if (curtime->tm_hour > 12)
     curtime->tm_hour -= 12;
   }

 // Strip to 10 minute scale
 if (!f_accurate)
  curtime->tm_min -= curtime->tm_min % 10;

 if (!f_quiet)
  printf ("[%02d:%02d]\n", curtime->tm_hour, curtime->tm_min);

 if (curtime->tm_hour < 21)
   {
    if (curtime->tm_hour > 3)	// In other case we use female numbers            
      {
       sprintf (szFilename, "%s%d.wav", soundpath, curtime->tm_hour);
       PlaySound (szFilename);
       goto donth10;
      }
   }
 else
   {
    sprintf (szFilename, "%s%d.wav", soundpath,
	     curtime->tm_hour - curtime->tm_hour % 10);
    PlaySound (szFilename);
   }

 switch (curtime->tm_hour % 10)
   {
   case 1:
   case 2:
    sprintf (szFilename, "%s%dmale.wav", soundpath, curtime->tm_hour % 10);
    PlaySound (szFilename);
    break;
   default:
    sprintf (szFilename, "%s%d.wav", soundpath, curtime->tm_hour % 10);
    PlaySound (szFilename);
   }
donth10:

 if (!fasttime || (fasttime && curtime->tm_hour == 1))
   {
    switch (curtime->tm_hour)
      {
      case 1:
      case 21:
       sprintf (szFilename, "%schas.wav", soundpath);
       break;
      case 2:
      case 3:
      case 4:
      case 22:
      case 23:
       sprintf (szFilename, "%schasa.wav", soundpath);
       break;
      default:
       sprintf (szFilename, "%schasov.wav", soundpath);
      }
    PlaySound (szFilename);
   }

 if (curtime->tm_min != 0)
   {
    if (curtime->tm_min < 20)
      {
       if (curtime->tm_min > 3)	// In other case we use female numbers            
	 {
	  sprintf (szFilename, "%s%d.wav", soundpath, curtime->tm_min);
	  PlaySound (szFilename);
	  goto dont10;
	 }
      }
    else
      {
       sprintf (szFilename, "%s%d.wav", soundpath,
		curtime->tm_min - curtime->tm_min % 10);
       PlaySound (szFilename);
      }

    switch (curtime->tm_min % 10)
      {
      case 0:
       break;
      case 1:
      case 2:
       sprintf (szFilename, "%s%dfem.wav", soundpath, curtime->tm_min % 10);
       PlaySound (szFilename);
       break;
      default:
       sprintf (szFilename, "%s%d.wav", soundpath, curtime->tm_min % 10);
       PlaySound (szFilename);
      }

  dont10:
    if (!fasttime)
      {
       if (curtime->tm_min % 10 == 1 && curtime->tm_min != 11)
	sprintf (szFilename, "%sminuta.wav", soundpath);
       else if (passes (5, 20) ||
		passes (25, 30) ||
		passes (35, 40) || passes (45, 50) || passes (55, 59))
	sprintf (szFilename, "%sminut.wav", soundpath);
       else
	sprintf (szFilename, "%sminuty.wav", soundpath);
       PlaySound (szFilename);
      }
   }
 else
   {
    // New hour
    sprintf (szFilename, "%srovno.wav", soundpath);
    PlaySound (szFilename);
   }

alldone:
 free (soundpath);
#ifndef EXTPLAYER
 CloseDSP ();
#endif
 return 0;
}
